@@ -119,14 +119,16 @@ normalize.n.models <- function(n.models, p, initprobs, method, bigmem) {
 #' @param na.action a function which indicates what should happen when the data
 #' contain NAs. The default is "na.omit".
 #' @param n.models number of models to sample either without replacement
-#' (method="BAS" or "MCMC+BAS") or with replacement (method="MCMC"). If NULL,
-#' BAS with method="BAS" will try to enumerate all 2^p models. If enumeration
-#' is not possible (memory or time) then a value should be supplied which
-#' controls the number of sampled models using 'n.models'.  With method="MCMC",
-#' sampling will stop once the min(n.models, MCMC.iterations) occurs so
-#' MCMC.iterations be significantly larger than n.models in order to explore the model space.
-#' On exit for method= "MCMC" this is the number of unique models that have
-#' been sampled with counts stored in the output as "freq".
+#' (method="BAS" or "MCMC+BAS") or initial number of models to sample with replacement (method="MCMC" or "AMCMC"). 
+#' If NULL,BAS with method="BAS" will try to enumerate/sample
+#' the min(2^p, 2^16). If 'n.models' > 2^25, the user should use 'bigmem = TRUE' to sample/enumerate
+#' 'n.models'.  With method="MCMC" or "AMCMC", 'n.models' controls the initial number of models with 
+#' the default for n.models = min(2000, 2^p).
+#' Sampling will stop once burnin.iterations + MCMC.iterations are exceeded 
+#' and n.models will be increased/decreased as needed to store the unique models sampled.
+#' On exit 'n.models' is the number of unique models that have
+#' been sampled.  For sampling with replacement (MCMC or AMCMC) the counts for 
+#' the number of times a models is sampled is stored in the output as "freq".
 #' @param prior prior distribution for regression coefficients.  Choices
 #' include
 #' \itemize{
@@ -185,7 +187,6 @@ normalize.n.models <- function(n.models, p, initprobs, method, bigmem) {
 #' }
 #' Note that Porwal & Raftery (2022) recommend alpha = sqrt(n) for the g-prior
 #' based on extensive range of simulations and examples for comparing BMA.
-#' This will become the default in the future.
 #' @param modelprior A function for a family of prior distribution on the models.  Choices
 #' include \code{\link{uniform}} \code{\link{Bernoulli}} or
 #' \code{\link{beta.binomial}}, \code{\link{tr.beta.binomial}},
@@ -237,8 +238,8 @@ normalize.n.models <- function(n.models, p, initprobs, method, bigmem) {
 #' based on factoring the proposal distribution as a product conditional probabilities
 #' estimated from the past draws. If 
 #' `importance.sampling = FALSE` this uses an adaptive independent Metropolis-Hasting
-#' algorithm, with if `importance.sampling = TRUE`  uses importance sampline 
-#' combined with Horiwitz-Thompson estimates of posterior model and inclusion
+#' algorithm, with if `importance.sampling = TRUE`  uses importance sampling 
+#' combined with Horwitz-Thompson estimates of posterior model and inclusion
 #' probabilities.
 #' }
 #' @param update number of iterations between potential updates of the sampling
@@ -254,9 +255,9 @@ normalize.n.models <- function(n.models, p, initprobs, method, bigmem) {
 #' random-walk Metropolis proposal; otherwise use a random "flip" move
 #' to propose swap a variable that is excluded with a variable in the model.
 #' @param burnin.iterations Number of burnin iterations for the MCMC sampler; the
-#' default is n.models*10 if not set by the user.
+#' default is p*25 if not set by the user.
 #' @param MCMC.iterations Number of iterations for the MCMC sampler; the
-#' default is n.models*10 if not set by the user.
+#' default is p*1000 if not set by the user.
 #' @param lambda Parameter in the AMCMC algorithm to insure positive definite 
 #' covariance of gammas for adaptive conditional probabilities prior based on prior degrees of freedom pseudo
 #' in Inverse-Wishart.  Default is set to p + 2.
@@ -274,6 +275,11 @@ normalize.n.models <- function(n.models, p, initprobs, method, bigmem) {
 #' See details in Clyde and Ghosh (2012).
 #' @param importance.sampling whether to use importance sampling or an independent
 #'  Metropolis-Hastings algorithm with sampling method="AMCMC" (see above).
+#' @param FPS Finite Population Sampling estimator for use with method="MCMC+BAS". Options include
+#' "none" (default) which uses the sum of marginal likelihoods times priors for sampled models
+#' to estimate the normalizing constant, or "Bayes_HT" 
+#' which uses an additional correction to the normalizing constant to account for unsampled models
+#' using sampling probabilities.  
 #' @param force.heredity  Logical variable to force all levels of a factor to be
 #' included together and to include higher order interactions only if lower
 #' order terms are included.  Currently supported with `method='MCMC'`
@@ -293,10 +299,20 @@ normalize.n.models <- function(n.models, p, initprobs, method, bigmem) {
 #' Currently coefficients that are not estimable are set to zero.  Use caution with
 #' interpreting BMA estimates of parameters.
 #' @param tol 1e-7 as
+#' @param GROW Logical variable to indicate that the output vectors in MCMC are growable.  
+#' Rather than allocate space based on `n.models`, the vectors will grow as needed
+#' if the number of unique models sampled exceeds the initial size of the allocated output vectors controlled by 
+#' `n.models.init`.  This is useful when `n.models` is unknown
+#' before reaching 'MCMC.iterations'.  Default is TRUE.
+#' @param expand variable to control how much to grow vectors with GROW = TRUE 
+#' if number of unique models exceeds the current size of the vectors. 
+#' The default is 1.25 times, which allows vectors to grow by 25 percent.
+#' @param n.models.init Initial size of output vectors if GROW = TRUE. The 
+#' default is `n.models = 2500`.
 #' @param bigmem Logical variable to indicate that there is access to
 #' large amounts of memory (physical or virtual) for enumeration
 #' with large model spaces, e.g. > 2^25. default; used in determining rank of 
-#' X^TX in cholesky decomposition with pivoting.
+#' X^TX in Cholesky decomposition with pivoting.
 #'
 #' @return \code{bas} returns an object of class \code{bas}
 #'
@@ -519,9 +535,13 @@ bas.lm <- function(formula,
                    thin = 1,
                    renormalize = FALSE, 
                    importance.sampling = FALSE,
+                   FPS = "none",
                    force.heredity = FALSE,
                    pivot = TRUE,
                    tol = 1e-7,
+                   GROW = TRUE,
+                   expand = 1.05,
+                   n.models.init = 2500,
                    bigmem = FALSE) {
   num.updates <- 10
   call <- match.call()
@@ -552,7 +572,14 @@ bas.lm <- function(formula,
     stop(paste("No available sampling method:", method))
   }
  
+  if (method == "BAS" & !GROW) {
+    method = "BAS_OLD"
+  }
+  if (method == "AMCMC" & !GROW) {
+    method = "AMCMC_OLD"
+  }
   
+ 
   # from lm
   mfall <- match.call(expand.dots = FALSE)
   m <- match(
@@ -664,18 +691,17 @@ bas.lm <- function(formula,
 
   if (is.null(n.models)) {
     n.models <- min(2^p, 2^16)
+    if (method == "MCMC")   n.models = min(n.models, n.models.init) 
+    if (method == "AMCMC" & !importance.sampling)  n.models = min(n.models, n.models.init) 
+
   }
   if (is.null(MCMC.iterations)) {
-    MCMC.iterations <- as.integer(n.models * 10)
+    MCMC.iterations <- as.integer(p * 1000)
   }
   if (is.null(burnin.iterations)){
-    burnin.iterations <- as.integer(n.models * 10)
+    burnin.iterations <- as.integer(p * 25)
     }
 
-  
-
-
-
 
   int <- TRUE # assume that an intercept is always included
 
@@ -765,9 +791,15 @@ bas.lm <- function(formula,
   bestmodel = as.integer(bestmodel)
   n.models <- normalize.n.models(n.models, p, prob,
                                   method, bigmem)
-  #  print(n.models)
+
   modelprior <- normalize.modelprior(modelprior, p)
 
+  if (method == "MCMC+BAS") {
+    FPS = as.integer(switch(FPS, 
+                            "none" = 0, 
+                            "Bayes_HT" = 1)
+                     )
+  }
 
   if (is.null(update)) {
     if (force.heredity) {  # do not update tree for BAS
@@ -787,10 +819,11 @@ bas.lm <- function(formula,
 
   if (is.null(lambda)) lambda = 0.0  # set default in C code
 
+ 
   #  sampleprobs = as.double(rep(0.0, n.models))
   result <- switch(
     method,
-    "BAS" = .Call(
+    "BAS_OLD" = .Call(
       C_sampleworep_new,
       Yvec,
       X,
@@ -809,6 +842,25 @@ bas.lm <- function(formula,
       Rtol = tol,
       PACKAGE = "BAS"
     ),
+    "BAS" = .Call(
+      C_sampleworep_grow,
+      Yvec,
+      X,
+      sqrt(weights),
+      prob,
+      n.models = as.integer(n.models),
+      incint = as.integer(int),
+      alpha = as.numeric(alpha),
+      method = as.integer(method.num),
+      modelprior = modelprior,
+      update = as.integer(update),
+      Rbestmodel = as.integer(bestmodel),
+      plocal = as.numeric(prob.local),
+      Rparents = parents,
+      Rpivot = pivot,
+      Rtol = tol,
+      PACKAGE = "BAS"
+    ),
     "MCMC+BAS" = .Call(
       C_mcmcbas,
       Yvec,
@@ -830,15 +882,16 @@ bas.lm <- function(formula,
       Rthin = as.integer(thin),
       Rparents = parents,
       Rpivot = pivot,
-      Rtol = tol
+      Rtol = tol,
+      RFPS = FPS
     ),
     "MCMC" = .Call(
-      C_mcmc_new,
+      C_mcmc_grow,
       Yvec,
       X,
       sqrt(weights),
       prob,
-      modeldim,
+      nModels = as.integer(n.models),
       incint = as.integer(int),
       alpha = as.numeric(alpha),
       method = as.integer(method.num),
@@ -853,9 +906,35 @@ bas.lm <- function(formula,
       Rthin = as.integer(thin),
       Rparents = parents,
       Rpivot = pivot,
-      Rtol = tol
+      Rtol = tol,
+      Rexpand = as.numeric(expand)
     ),
     "AMCMC" = .Call(
+      C_amcmc_grow,
+      Yvec,
+      X,
+      sqrt(weights),
+      prob,
+      RnModels = as.integer(n.models),
+      incint = as.integer(int),
+      alpha = as.numeric(alpha),
+      method = as.integer(method.num),
+      modelprior = modelprior,
+      update = as.integer(update),
+      Rbestmodel = as.integer(bestmodel),
+      plocal = as.numeric(1.0 - prob.rw),
+      as.integer(burnin.iterations),
+      as.integer(MCMC.iterations),
+      as.numeric(lambda),
+      as.numeric(delta),
+      Rthin = as.integer(thin),
+      Rparents = parents,
+      Rpivot = pivot,
+      Rtol = tol,
+      RIS = importance.sampling,
+      Rexpand = expand
+    ),
+    "AMCMC_OLD" = .Call(
       C_amcmc,
       Yvec,
       X,
@@ -944,7 +1023,7 @@ bas.lm <- function(formula,
   }
   
   if (importance.sampling) renormalize = TRUE # do not use MCMC probs and use HT
-  if (method == "MCMC" || method == "AMCMC") {
+  if (method == "MCMC" || method == "AMCMC" || method == "AMCMC_OLD") {
     result$n.models <- result$n.Unique
     result$postprobs.MCMC <- result$freq / sum(result$freq)
 
