@@ -1,3 +1,8 @@
+// Copyright (c) 2024 Merlise Clyde and contributors to BAS. All rights reserved.
+// This work is licensed under a GNU GENERAL PUBLIC LICENSE Version 3.0
+// License text is available at https://www.gnu.org/licenses/gpl-3.0.html
+// SPDX-License-Identifier: GPL-3.0
+//
 #include "bas.h"
 
 
@@ -217,7 +222,9 @@ ete = 0.0;
 		ete = F77_NAME(ddot)(&p, &coefficients[0], &inc, &XtY[0], &inc);
 
 //	Rprintf("YtY= %lf ete = %lf rank = %d,  p = %d", *mse, ete, rank, p);
-	if ( n <= p ) {
+
+//	Issue #96
+	if ( n <= rank ) {
 	  *mse = 0.0;
 	}
 	else {
