@@ -14,6 +14,8 @@ SEXP amcmc(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeldim,
               SEXP Rthin, SEXP Rparents, SEXP Rpivot, SEXP Rtol, SEXP RIS)
 {
 	int nProtected = 0;
+  int *counts;
+  
 	SEXP RXwork = PROTECT(duplicate(X)); nProtected++;
 	SEXP RYwork = PROTECT(duplicate(Y)); nProtected++;
 	int nModels=LENGTH(Rmodeldim);
@@ -21,7 +23,79 @@ SEXP amcmc(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeldim,
 	double tol = REAL(Rtol)[0];
 
 	
-
+	SEXP ANS = PROTECT(allocVector(VECSXP, 16)); ++nProtected;
+	SEXP ANS_names = PROTECT(allocVector(STRSXP, 16)); ++nProtected;
+	
+	SEXP Rprobs = duplicate(Rprobinit); 
+	SET_VECTOR_ELT(ANS, 0, Rprobs);
+	SET_STRING_ELT(ANS_names, 0, mkChar("probne0"));
+	
+	SEXP modelspace = allocVector(VECSXP, nModels); 
+	SET_VECTOR_ELT(ANS, 1, modelspace);
+	SET_STRING_ELT(ANS_names, 1, mkChar("which"));
+	
+	SEXP Rlogmarg = allocVector(REALSXP, nModels); 
+	SET_VECTOR_ELT(ANS, 2, Rlogmarg);
+	SET_STRING_ELT(ANS_names, 2, mkChar("logmarg"));
+	
+	SEXP modelprobs = allocVector(REALSXP, nModels);  
+	SET_VECTOR_ELT(ANS, 3, modelprobs);
+	SET_STRING_ELT(ANS_names, 3, mkChar("postprobs"));
+	
+	SEXP priorprobs = allocVector(REALSXP, nModels); 
+	SET_VECTOR_ELT(ANS, 4, priorprobs);
+	SET_STRING_ELT(ANS_names, 4, mkChar("priorprobs"));
+	
+	SEXP sampleprobs = allocVector(REALSXP, nModels); 
+	SET_VECTOR_ELT(ANS, 5, sampleprobs);
+	SET_STRING_ELT(ANS_names, 5, mkChar("sampleprobs"));
+	
+	SEXP mse = allocVector(REALSXP, nModels); 
+	SET_VECTOR_ELT(ANS, 6, mse);
+	SET_STRING_ELT(ANS_names, 6, mkChar("mse"));
+	
+	SEXP beta = allocVector(VECSXP, nModels); 
+	SET_VECTOR_ELT(ANS, 7, beta);
+	SET_STRING_ELT(ANS_names, 7, mkChar("mle"));
+	
+	SEXP se = allocVector(VECSXP, nModels);
+	SET_VECTOR_ELT(ANS, 8, se);
+	SET_STRING_ELT(ANS_names, 8, mkChar("mle.se"));
+	
+	SEXP shrinkage = allocVector(REALSXP, nModels); 
+	SET_VECTOR_ELT(ANS, 9, shrinkage);
+	SET_STRING_ELT(ANS_names, 9, mkChar("shrinkage"));
+	
+	SEXP modeldim =  allocVector(INTSXP, nModels); 
+	SET_VECTOR_ELT(ANS, 10, modeldim);
+	SET_STRING_ELT(ANS_names, 10, mkChar("size"));
+	
+	SEXP R2 = allocVector(REALSXP, nModels); 
+	SET_VECTOR_ELT(ANS, 11, R2);
+	SET_STRING_ELT(ANS_names, 11, mkChar("R2"));
+	
+	SEXP rank = allocVector(INTSXP, nModels); 
+	SET_VECTOR_ELT(ANS, 12, rank);
+	SET_STRING_ELT(ANS_names, 12, mkChar("rank"));
+	
+	SEXP Rcounts =  allocVector(INTSXP, nModels); 
+	counts = INTEGER(Rcounts);
+	memset(counts, 0, nModels * sizeof(int));
+	SET_VECTOR_ELT(ANS, 13, Rcounts);
+	SET_STRING_ELT(ANS_names, 13, mkChar("freq"));
+	
+	SEXP MCMCprobs= duplicate(Rprobinit);
+	SET_VECTOR_ELT(ANS, 14, MCMCprobs);
+	SET_STRING_ELT(ANS_names, 14, mkChar("probne0.MCMC"));
+	
+	SEXP NumUnique = allocVector(INTSXP, 1); 
+	SET_VECTOR_ELT(ANS, 15, NumUnique);
+	SET_STRING_ELT(ANS_names, 15, mkChar("n.Unique"));
+	
+	setAttrib(ANS, R_NamesSymbol, ANS_names);
+	
+	
+/*
 	SEXP ANS = PROTECT(allocVector(VECSXP, 16)); ++nProtected;
 	SEXP ANS_names = PROTECT(allocVector(STRSXP, 16)); ++nProtected;
 	SEXP Rprobs = PROTECT(duplicate(Rprobinit)); ++nProtected;
@@ -31,23 +105,23 @@ SEXP amcmc(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeldim,
 	SEXP modelspace = PROTECT(allocVector(VECSXP, nModels)); ++nProtected;
 	SEXP rank = PROTECT(allocVector(INTSXP, nModels)); ++nProtected;
 	SEXP modeldim =  PROTECT(duplicate(Rmodeldim)); ++nProtected;
-	SEXP counts =  PROTECT(duplicate(Rmodeldim)); ++nProtected;
+	SEXP counts =  PROTECT(allocVector(INTSXP, nModels)); ++nProtected;
 	SEXP beta = PROTECT(allocVector(VECSXP, nModels)); ++nProtected;
 	SEXP se = PROTECT(allocVector(VECSXP, nModels)); ++nProtected;
 	SEXP mse = PROTECT(allocVector(REALSXP, nModels)); ++nProtected;
 	SEXP modelprobs = PROTECT(allocVector(REALSXP, nModels)); ++nProtected;
 	SEXP priorprobs = PROTECT(allocVector(REALSXP, nModels)); ++nProtected;
-	SEXP logmarg = PROTECT(allocVector(REALSXP, nModels)); ++nProtected;
+	SEXP Rlogmarg = PROTECT(allocVector(REALSXP, nModels)); ++nProtected;
 	SEXP sampleprobs = PROTECT(allocVector(REALSXP, nModels)); ++nProtected;
 	SEXP NumUnique = PROTECT(allocVector(INTSXP, 1)); ++nProtected;
 
-	
+*/	
 
 	
-	Rprintf("Allocating Space for %d Models AMCMC\n", nModels) ;
+	// Rprintf("Allocating Space for %d Models AMCMC\n", nModels) ;
 	double *Xwork, *Ywork,*wts, *probs, shrinkage_m,
 		mse_m, MH=0.0, prior_m=1.0,
-		R2_m, RSquareFull, logmargy, postold, postnew;
+		R2_m, RSquareFull, logmarg_m, postold, postnew;
 
 	double *SSgam, *Cov, *priorCov, *marg_probs, one=1.0, wt = 1.0;
 	int i,j, m, n, pmodel_old, *model_m, *bestmodel, rank_m;
@@ -55,12 +129,12 @@ SEXP amcmc(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeldim,
 	int print = 1;
   bool IS = LOGICAL(RIS)[0];
   
-	Rprintf("AMCMC\n") ;
+//	Rprintf("AMCMC\n") ;
 	
 	//get dimsensions of all variables
 	int nobs = LENGTH(Y);
 	int p = INTEGER(getAttrib(X,R_DimSymbol))[1];
-	int k = LENGTH(modelprobs);
+//	int k = LENGTH(modelprobs);
 	double lambda=REAL(LAMBDA)[0];
 	double delta = REAL(DELTA)[0];
 	double alpha = REAL(Ralpha)[0];
@@ -143,20 +217,21 @@ SEXP amcmc(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeldim,
 	PROTECT(Rse_m = NEW_NUMERIC(pmodel));
 
 	model_m = GetModel_m(Rmodel_m, model, p);
-	//evaluate logmargy and shrinkage
+	//evaluate logmarg_m and shrinkage
 
 	R2_m = FitModel(Rcoef_m, Rse_m, XtY, XtX, model_m, XtYwork, XtXwork, yty, SSY, pmodel,
                  p, nobs, m, &mse_m, &rank_m, pivot, tol);
 	INTEGER(rank)[0] = rank_m;
 
-	gexpectations(p, rank_m, nobs, R2_m, alpha, INTEGER(method)[0], RSquareFull, SSY, &logmargy,
+	gexpectations(p, rank_m, nobs, R2_m, alpha, INTEGER(method)[0], RSquareFull, SSY, &logmarg_m,
                &shrinkage_m);
 
 
 	prior_m  = compute_prior_probs(model,pmodel,p, modelprior, noInclusionIs1);
 	if (prior_m == 0.0)  error("initial model has 0 prior probabilty\n");
-	SetModel2(logmargy, shrinkage_m, prior_m, sampleprobs, logmarg, shrinkage, priorprobs, m);
-	SetModel(Rcoef_m, Rse_m, Rmodel_m, mse_m, R2_m,	beta, se, modelspace, mse, R2, m);
+//	SetModel2(logmarg_m, shrinkage_m, prior_m, sampleprobs, Rlogmarg, shrinkage, priorprobs, m);
+	SetModel_lm(logmarg_m, shrinkage_m, prior_m, sampleprobs, Rlogmarg, shrinkage, priorprobs,
+              Rcoef_m, Rse_m, Rmodel_m, mse_m, R2_m,	beta, se, modelspace, mse, R2, m);
 
 	int nUnique=0, newmodel=0, nsamples=0;
 	double *real_model = vecalloc(n);
@@ -165,17 +240,17 @@ SEXP amcmc(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeldim,
 	int new_loc;
 	pmodel_old = pmodel;
 	nUnique=1;
-	INTEGER(counts)[0] = 1;
-	postold =  REAL(logmarg)[m] + log(REAL(priorprobs)[m]);
+	INTEGER(Rcounts)[0] = 1;
+	postold =  REAL(Rlogmarg)[m] + log(REAL(priorprobs)[m]);
 	memcpy(modelold, model, sizeof(int)*p);
 	m = 0;
 	int *varin= ivecalloc(p);
 	int *varout= ivecalloc(p);
 	double problocal = REAL(plocal)[0];
 	
-	Rprintf("using MCMC sampling - initialize\n");
+//	Rprintf("using MCMC sampling - initialize\n");
 	
-	while (nUnique < k && m < INTEGER(BURNIN_Iterations)[0]) {
+	while (nUnique < nModels && m < INTEGER(BURNIN_Iterations)[0]) {
 
 	  memcpy(model, modelold, sizeof(int)*p);
 		pmodel =  n_sure;
@@ -215,14 +290,14 @@ SEXP amcmc(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeldim,
 
 		    R2_m = FitModel(Rcoef_m, Rse_m, XtY, XtX, model_m, XtYwork, XtXwork, yty, SSY, pmodel, p, nobs, m, &mse_m,
                       &rank_m, pivot, tol);
-		    gexpectations(p, rank_m, nobs, R2_m, alpha, INTEGER(method)[0], RSquareFull, SSY, &logmargy, &shrinkage_m);
+		    gexpectations(p, rank_m, nobs, R2_m, alpha, INTEGER(method)[0], RSquareFull, SSY, &logmarg_m, &shrinkage_m);
 
-		    postnew = logmargy + log(prior_m);
+		    postnew = logmarg_m + log(prior_m);
 		    MH *= exp(postnew - postold);
 		  }}
 		else {
 		  new_loc = branch->where;
-		  postnew =  REAL(logmarg)[new_loc] +
+		  postnew =  REAL(Rlogmarg)[new_loc] +
 		             log(REAL(priorprobs)[new_loc]);
 		  MH *=  exp(postnew - postold);
 		}
@@ -236,10 +311,12 @@ SEXP amcmc(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeldim,
 		    insert_model_tree(tree, vars, n, model, nUnique);
 		    INTEGER(modeldim)[nUnique] = pmodel;
 		    INTEGER(rank)[nUnique] = rank_m;
+		    INTEGER(Rcounts)[nUnique] = 0;
         
 		    //record model data
-		    SetModel2(logmargy, shrinkage_m, prior_m, sampleprobs, logmarg, shrinkage, priorprobs, nUnique);
-		    SetModel(Rcoef_m, Rse_m, Rmodel_m, mse_m, R2_m,	beta, se, modelspace, mse, R2,nUnique);
+//		    SetModel2(logmarg_m, shrinkage_m, prior_m, sampleprobs, Rlogmarg, shrinkage, priorprobs, nUnique);
+		    SetModel_lm(logmarg_m, shrinkage_m, prior_m, sampleprobs, Rlogmarg, shrinkage, priorprobs,
+                    Rcoef_m, Rse_m, Rmodel_m, mse_m, R2_m,	beta, se, modelspace, mse, R2,nUnique);
 
 		    ++nUnique;
 		    }
@@ -257,7 +334,7 @@ SEXP amcmc(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeldim,
 
 		if ( (m % thin) == 0) {
 
-		  INTEGER(counts)[old_loc] += 1;
+		  INTEGER(Rcounts)[old_loc] += 1;
 
 		  for (i = 0; i < n; i++) {
 		    // store in tree order //
@@ -275,7 +352,7 @@ SEXP amcmc(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeldim,
 	
 	// Compute marginal probabilities  
 	mcurrent = nUnique;
-	compute_modelprobs(modelprobs, logmarg, priorprobs,mcurrent);
+	compute_modelprobs(modelprobs, Rlogmarg, priorprobs,mcurrent);
 	compute_margprobs(modelspace, modeldim, modelprobs, probs, mcurrent, p);        
 	for (i = 0; i < n; i++) {
 	  marg_probs[i] = wt*(REAL(MCMCprobs)[vars[i].index]/ (double) nsamples) + 
@@ -304,10 +381,10 @@ SEXP amcmc(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeldim,
 	*/
   // now use AMCMC
   
-  Rprintf("Now start AMCMC with %d nUnique models out of %d at it %d\n", nUnique, k, m);
+  // Rprintf("Now start AMCMC with %d nUnique models out of %d at it %d\n", nUnique, k, m);
   if (IS) thin = 1; // no need to thin
   
-  while (nUnique < k && m < (INTEGER(BURNIN_Iterations)[0] + INTEGER(MCMC_Iterations)[0])) {
+  while (nUnique < nModels && m < (INTEGER(BURNIN_Iterations)[0] + INTEGER(MCMC_Iterations)[0])) {
     
     memcpy(model, modelold, sizeof(int)*p);
     pmodel =  n_sure;
@@ -330,7 +407,7 @@ SEXP amcmc(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeldim,
       pmodel  += bit;
     }
     
-    MH = 1.0;   // if using IS set MH to 1 to alwas accept
+    MH = 1.0;   // if using IS set MH to 1 to always accept
     if (newmodel == 1) {
       prior_m = compute_prior_probs(model,pmodel,p, modelprior, noInclusionIs1);
       if (prior_m == 0.0 || pigammanew == 0.0) {
@@ -345,14 +422,14 @@ SEXP amcmc(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeldim,
         
         R2_m = FitModel(Rcoef_m, Rse_m, XtY, XtX, model_m, XtYwork, XtXwork, yty, SSY, pmodel, p, nobs, m, &mse_m,
                         &rank_m, pivot, tol);
-        gexpectations(p, rank_m, nobs, R2_m, alpha, INTEGER(method)[0], RSquareFull, SSY, &logmargy, &shrinkage_m);
+        gexpectations(p, rank_m, nobs, R2_m, alpha, INTEGER(method)[0], RSquareFull, SSY, &logmarg_m, &shrinkage_m);
         
-        postnew = logmargy + log(prior_m);
+        postnew = logmarg_m + log(prior_m);
         if (!IS) MH = exp(postnew - postold + log(pigammaold) - log(pigammanew));
       }}
     else {
       new_loc = branch->where;
-      postnew =  REAL(logmarg)[new_loc] +
+      postnew =  REAL(Rlogmarg)[new_loc] +
         log(REAL(priorprobs)[new_loc]);
       if (!IS) MH =  exp(postnew - postold + log(pigammaold) - log(pigammanew));
     }
@@ -366,10 +443,11 @@ SEXP amcmc(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeldim,
           insert_model_tree(tree, vars, n, model, nUnique);
           INTEGER(modeldim)[nUnique] = pmodel;
           INTEGER(rank)[nUnique] = rank_m;
-          
+          INTEGER(Rcounts)[nUnique] = 0;
           //record model data
-          SetModel2(logmargy, shrinkage_m, prior_m, sampleprobs, logmarg, shrinkage, priorprobs, nUnique);
-          SetModel(Rcoef_m, Rse_m, Rmodel_m, mse_m, R2_m,	beta, se, modelspace, mse, R2,nUnique);
+//          SetModel2(logmarg_m, shrinkage_m, prior_m, sampleprobs, Rlogmarg, shrinkage, priorprobs, nUnique);
+          SetModel_lm(logmarg_m, shrinkage_m, prior_m, sampleprobs, Rlogmarg, shrinkage, priorprobs,
+                      Rcoef_m, Rse_m, Rmodel_m, mse_m, R2_m,	beta, se, modelspace, mse, R2,nUnique);
           
           ++nUnique;
         }
@@ -388,7 +466,7 @@ SEXP amcmc(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeldim,
     
     if ( (m % thin) == 0) {
       
-      INTEGER(counts)[old_loc] += 1;
+      INTEGER(Rcounts)[old_loc] += 1;
       REAL(sampleprobs)[old_loc] = pigammaold;
       for (i = 0; i < n; i++) {
         real_model[i] = (double) modelold[vars[i].index];
@@ -408,21 +486,37 @@ SEXP amcmc(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeldim,
 	// Compute marginal probabilities
 	mcurrent = nUnique;
   if (IS && (INTEGER(MCMC_Iterations)[0] > 0)) {
-    compute_modelprobs_HT(modelprobs, logmarg, priorprobs, sampleprobs,
+    compute_modelprobs_HT(modelprobs, Rlogmarg, priorprobs, sampleprobs,
                           mcurrent, INTEGER(MCMC_Iterations)[0]);
                   }
-	else compute_modelprobs(modelprobs, logmarg, priorprobs,mcurrent);
+	else compute_modelprobs(modelprobs, Rlogmarg, priorprobs,mcurrent);
 	compute_margprobs(modelspace, modeldim, modelprobs, probs, mcurrent, p);
 
 	INTEGER(NumUnique)[0] = nUnique;
 
 	SET_VECTOR_ELT(ANS, 0, Rprobs);
-	SET_STRING_ELT(ANS_names, 0, mkChar("probne0"));
-
-//	Rprintf("truncate vectors/n");
+  
+	if (nUnique < nModels) {
+	  SET_VECTOR_ELT(ANS, 1, resizeVector(modelspace, nUnique));
+	  SET_VECTOR_ELT(ANS, 2, resizeVector(Rlogmarg, nUnique));
+	  SET_VECTOR_ELT(ANS, 3, resizeVector(modelprobs, nUnique));
+	  SET_VECTOR_ELT(ANS, 4, resizeVector(priorprobs, nUnique));
+	  SET_VECTOR_ELT(ANS, 5, resizeVector(sampleprobs, nUnique));
+	  SET_VECTOR_ELT(ANS, 6, resizeVector(mse, nUnique));
+	  SET_VECTOR_ELT(ANS, 7, resizeVector(beta, nUnique));
+	  SET_VECTOR_ELT(ANS, 8, resizeVector(se, nUnique));
+	  SET_VECTOR_ELT(ANS, 9, resizeVector(shrinkage, nUnique));
+	  SET_VECTOR_ELT(ANS, 10, resizeVector(modeldim, nUnique));
+	  SET_VECTOR_ELT(ANS, 11, resizeVector(R2, nUnique));
+	  SET_VECTOR_ELT(ANS, 12, resizeVector(rank, nUnique));
+	  SET_VECTOR_ELT(ANS, 13, resizeVector(Rcounts, nUnique));
+	}	 
+	
+	
+/*	Rprintf("truncate vectors/n");
 	if (nUnique < nModels) {
 	  SETLENGTH(modelspace, nUnique);
-	  SETLENGTH(logmarg, nUnique);
+	  SETLENGTH(Rlogmarg, nUnique);
 	  SETLENGTH(modelprobs, nUnique);
 	  SETLENGTH(priorprobs, nUnique);
 	  SETLENGTH(sampleprobs, nUnique);
@@ -438,7 +532,7 @@ SEXP amcmc(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeldim,
 	SET_VECTOR_ELT(ANS, 1, modelspace);
 	SET_STRING_ELT(ANS_names, 1, mkChar("which"));
 
-	SET_VECTOR_ELT(ANS, 2, logmarg);
+	SET_VECTOR_ELT(ANS, 2, Rlogmarg);
 	SET_STRING_ELT(ANS_names, 2, mkChar("logmarg"));
 
 	SET_VECTOR_ELT(ANS, 3, modelprobs);
@@ -481,6 +575,7 @@ SET_VECTOR_ELT(ANS, 15, NumUnique);
 SET_STRING_ELT(ANS_names, 15, mkChar("n.Unique"));
 
 setAttrib(ANS, R_NamesSymbol, ANS_names);
+*/
 
 	PutRNGstate();
     UNPROTECT(nProtected);
