@@ -20,13 +20,13 @@ SEXP mcmcbas(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeldim,
              SEXP incint, SEXP Ralpha,SEXP method, SEXP modelprior, SEXP Rupdate, 
              SEXP Rbestmodel,  SEXP plocal,
              SEXP BURNIN_Iterations, SEXP MCMC_Iterations, SEXP LAMBDA, SEXP DELTA,
-             SEXP Rthin, SEXP Rparents, SEXP Rpivot, SEXP Rtol)
+             SEXP Rthin, SEXP Rparents, SEXP Rpivot, SEXP Rtol, SEXP RFPS)
 {
 
   int nProtected = 0;
   SEXP RXwork = PROTECT(duplicate(X)); nProtected++; 
   SEXP RYwork = PROTECT(duplicate(Y));  nProtected++; 
-  int nModels=LENGTH(Rmodeldim);
+  int nModels = LENGTH(Rmodeldim);
   int pivot = LOGICAL(Rpivot)[0];
   double tol = REAL(Rtol)[0];
   int nUnique=0;
@@ -47,13 +47,13 @@ SEXP mcmcbas(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeldim,
   SEXP mse = PROTECT(allocVector(REALSXP, nModels)); ++nProtected;
   SEXP modelprobs = PROTECT(allocVector(REALSXP, nModels)); ++nProtected;
   SEXP priorprobs = PROTECT(allocVector(REALSXP, nModels)); ++nProtected;
-  SEXP logmarg = PROTECT(allocVector(REALSXP, nModels)); ++nProtected;
+  SEXP Rlogmarg = PROTECT(allocVector(REALSXP, nModels)); ++nProtected;
   SEXP sampleprobs = PROTECT(allocVector(REALSXP, nModels)); ++nProtected;
   SEXP NumUnique = PROTECT(allocVector(INTSXP, 1)); ++nProtected;
   
 
   double *Xwork, *Ywork, *wts, *probs, shrinkage_m, mse_m, R2_m, RSquareFull,
-          Rbestmarg, logmargy,  MH=0.0, postold, postnew;
+          Rbestmarg, logmarg_m,  MH=0.0, postold, postnew;
   int  i, *model_m, *bestmodel, rank_m, pmodel_old;
   int mcurrent, n_sure;
   double  mod, rem;
@@ -63,7 +63,7 @@ SEXP mcmcbas(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeldim,
   /* get dimsensions of all variables */
   int nobs = LENGTH(Y);
   int p = INTEGER(getAttrib(X,R_DimSymbol))[1];
-  int k = LENGTH(modelprobs);
+
   double alpha = REAL(Ralpha)[0];
   int thin = INTEGER(Rthin)[0];
   SEXP Rbestmodel_new = PROTECT(duplicate(Rbestmodel)); nProtected++;
@@ -120,7 +120,7 @@ SEXP mcmcbas(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeldim,
 
   int m = 0;
   bestmodel = INTEGER(Rbestmodel_new);
-  REAL(logmarg)[m] = 0.0;
+  REAL(Rlogmarg)[m] = 0.0;
   INTEGER(modeldim)[m] = 0;
   
   for (i = n; i < p; i++)  {
@@ -157,19 +157,20 @@ SEXP mcmcbas(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeldim,
                     p, nobs, m, &mse_m, &rank_m, pivot, tol);
     INTEGER(rank)[0] = rank_m;
     
-    gexpectations(p, rank_m, nobs, R2_m, alpha, INTEGER(method)[0], RSquareFull, SSY, &logmargy,
+    gexpectations(p, rank_m, nobs, R2_m, alpha, INTEGER(method)[0], RSquareFull, SSY, &logmarg_m,
                   &shrinkage_m);
     
     
     double prior_m  = compute_prior_probs(model,pmodel,p, modelprior, noInclusionIs1);
     if (prior_m == 0.0)  error("initial model has 0 prior probabilty\n");
   
-    SetModel2(logmargy, shrinkage_m, prior_m, sampleprobs, logmarg, shrinkage, priorprobs, m);
-    SetModel(Rcoef_m, Rse_m, Rmodel_m, mse_m, R2_m,	beta, se, modelspace, mse, R2, m);
+//    SetModel2(logmarg_m, shrinkage_m, prior_m, sampleprobs, Rlogmarg, shrinkage, priorprobs, m);
+    SetModel_lm(logmarg_m, shrinkage_m, prior_m, sampleprobs, Rlogmarg, shrinkage, priorprobs,
+                Rcoef_m, Rse_m, Rmodel_m, mse_m, R2_m,	beta, se, modelspace, mse, R2, m);
 
    
 //    UNPROTECT(3);
-    Rbestmarg = REAL(logmarg)[m];
+    Rbestmarg = REAL(Rlogmarg)[m];
     
     nUnique=1;
     int newmodel=0, nsamples=0;
@@ -182,7 +183,7 @@ SEXP mcmcbas(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeldim,
     pmodel_old = pmodel;
   
     INTEGER(counts)[0] = 1;
-    postold =  REAL(logmarg)[m] + log(REAL(priorprobs)[m]);
+    postold =  REAL(Rlogmarg)[m] + log(REAL(priorprobs)[m]);
     
 
     int *varin= ivecalloc(p);
@@ -196,7 +197,7 @@ SEXP mcmcbas(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeldim,
 
   m = 0;
   
-  while ( m < INTEGER(BURNIN_Iterations)[0] && nUnique < k) {
+  while ( m < INTEGER(BURNIN_Iterations)[0] && nUnique < nModels) {
     
     memcpy(model, modelold, sizeof(int)*p);
     pmodel =  n_sure;
@@ -237,14 +238,14 @@ SEXP mcmcbas(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeldim,
         
         R2_m = FitModel(Rcoef_m, Rse_m, XtY, XtX, model_m, XtYwork, XtXwork, yty, SSY, pmodel, p, nobs, m, &mse_m,
                         &rank_m, pivot, tol);
-        gexpectations(p, rank_m, nobs, R2_m, alpha, INTEGER(method)[0], RSquareFull, SSY, &logmargy, &shrinkage_m);
+        gexpectations(p, rank_m, nobs, R2_m, alpha, INTEGER(method)[0], RSquareFull, SSY, &logmarg_m, &shrinkage_m);
         
-        postnew = logmargy + log(prior_m);
+        postnew = logmarg_m + log(prior_m);
         MH *= exp(postnew - postold);
       }}
     else {
       new_loc = branch->where;
-      postnew =  REAL(logmarg)[new_loc] +
+      postnew =  REAL(Rlogmarg)[new_loc] +
         log(REAL(priorprobs)[new_loc]);
       MH *=  exp(postnew - postold);
     }
@@ -259,10 +260,11 @@ SEXP mcmcbas(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeldim,
           INTEGER(modeldim)[nUnique] = pmodel;
           INTEGER(rank)[nUnique] = rank_m;
           
-          //record model data
-          SetModel2(logmargy, shrinkage_m, prior_m, sampleprobs, logmarg, shrinkage, priorprobs, nUnique);
-          SetModel(Rcoef_m, Rse_m, Rmodel_m, mse_m, R2_m,	beta, se, modelspace, mse, R2,nUnique);
           
+          //record model data
+//          SetModel2(logmarg_m, shrinkage_m, prior_m, sampleprobs, Rlogmarg, shrinkage, priorprobs, nUnique);
+          SetModel_lm(logmarg_m, shrinkage_m, prior_m, sampleprobs, Rlogmarg, shrinkage, priorprobs,
+                      Rcoef_m, Rse_m, Rmodel_m, mse_m, R2_m,	beta, se, modelspace, mse, R2,nUnique);
           ++nUnique;
         }
         else UNPROTECT(3);
@@ -297,17 +299,15 @@ SEXP mcmcbas(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeldim,
   // Compute MCMC inclusion probabilities
 
   for (i = 0; i < n; i++) {
-     REAL(MCMCprobs)[vars[i].index] /= (double) m;
+     REAL(MCMCprobs)[vars[i].index] /= (double) nsamples;
   }
   // Rprintf("\n Num Unique models %d  in %d MCMC iterations \n", nUnique, m);
 
 
 // Compute marginal probabilities
   mcurrent = nUnique - 1;
-  compute_modelprobs(modelprobs, logmarg, priorprobs,mcurrent);
-  compute_margprobs(modelspace, modeldim, modelprobs, probs, mcurrent, p);
-
-
+  compute_modelprobs(modelprobs, Rlogmarg, priorprobs,mcurrent);
+  compute_margprobs(modelspace, modeldim, modelprobs, probs, nUnique, p);
 
 //  Now sample W/O Replacement
 // Rprintf("Sample w/out Replacement Now \n", nUnique);
@@ -316,14 +316,14 @@ SEXP mcmcbas(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeldim,
 
  m = mcurrent; 
  
- if (m < k && INTEGER(MCMC_Iterations)[0] > 0) {
-  if (update_probs(probs, vars, mcurrent, k, p) == 1) {
-     // Rprintf("updating tree for SWOR\n");
-     update_tree(modelspace, tree, modeldim, vars, k,p,n,mcurrent, modelwork);
-     // Rprintf("Done!\n");
-  }}
+ //    Rprintf("updating tree for SWOR\n");
+  if (m < nModels ) {
+  if (update_probs(probs, vars, mcurrent, nModels, p) == 1) {
+     update_tree(modelspace, tree, modeldim, vars, nModels, p, n, mcurrent, modelwork);
+    }
+  }
  
-  for (m = nUnique;  m < k && lessThanOne(pigamma[0]); m++) {
+  for (m = nUnique;  m < nModels && lessThanOne(pigamma[0]); m++) {
     INTEGER(modeldim)[m] = n_sure;
 
     branch = tree;
@@ -347,36 +347,35 @@ SEXP mcmcbas(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeldim,
                     pmodel, p, nobs, m, &mse_m, &rank_m, pivot, tol);
     INTEGER(rank)[m] = rank_m;
     // initialize
-    logmargy= 0.0;
+    logmarg_m= 0.0;
     shrinkage_m = 1.0;
-    gexpectations(p, rank_m, nobs, R2_m, alpha, INTEGER(method)[0], RSquareFull, SSY, &logmargy, &shrinkage_m);
+    gexpectations(p, rank_m, nobs, R2_m, alpha, INTEGER(method)[0], RSquareFull, SSY, &logmarg_m, &shrinkage_m);
     //    Rprintf("rank %d dim %d\n", rank_m, pmodel);
-    //		gexpectations(p, pmodel, nobs, R2_m, alpha, INTEGER(method)[0], RSquareFull, SSY, &logmargy, &shrinkage_m);
+    //		gexpectations(p, pmodel, nobs, R2_m, alpha, INTEGER(method)[0], RSquareFull, SSY, &logmarg_m, &shrinkage_m);
     
     prior_m = compute_prior_probs(model,pmodel,p, modelprior, noInclusionIs1);
-    SetModel2(logmargy, shrinkage_m, prior_m, sampleprobs, logmarg, shrinkage, priorprobs, m);
-    SetModel_lm(Rcoef_m, Rse_m, Rmodel_m, mse_m, R2_m,	beta, se, modelspace, mse, R2,m);
-    UNPROTECT(3);
+//    SetModel2(logmarg_m, shrinkage_m, prior_m, sampleprobs, Rlogmarg, shrinkage, priorprobs, m);
+    SetModel_lm(logmarg_m, shrinkage_m, prior_m, sampleprobs, Rlogmarg, shrinkage, priorprobs,
+                Rcoef_m, Rse_m, Rmodel_m, mse_m, R2_m,	beta, se, modelspace, mse, R2,m);
+
+//   REAL(sampleprobs)[m] = compute_sample_probs_bernoulli(Rprobs, model, p);
     
-    REAL(sampleprobs)[m] = pigamma[0];
-    
-    //update best model
-    if (REAL(logmarg)[m] > Rbestmarg) {
+    if (REAL(Rlogmarg)[m] > Rbestmarg) {
       for (i=0; i < p; i++) {
         bestmodel[i] = model[i];
       }
-      Rbestmarg = REAL(logmarg)[m];
+      Rbestmarg = REAL(Rlogmarg)[m];
     }
 
     if (m > 1) {
       rem = modf((double) m/(double) update, &mod);
       if (rem  == 0.0) {
       	mcurrent = m;
-	      compute_modelprobs(modelprobs, logmarg, priorprobs,mcurrent);
+	      compute_modelprobs(modelprobs, Rlogmarg, priorprobs,mcurrent);
 	      compute_margprobs(modelspace, modeldim, modelprobs, probs, mcurrent, p);
-	      if (update_probs(probs, vars, mcurrent, k, p) == 1) {
+	      if (update_probs(probs, vars, mcurrent,nModels, p) == 1) {
         	  // Rprintf("Updating Model Tree %d \n", m);
-	           update_tree(modelspace, tree, modeldim, vars, k, p, n ,mcurrent,
+	           update_tree(modelspace, tree, modeldim, vars,nModels, p, n ,mcurrent,
                          modelwork);
 	          // Rprintf("Done with update\n");
 	        }
@@ -388,29 +387,37 @@ SEXP mcmcbas(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeldim,
   }
  
  
- /*. for when add heridity 
- if (m < k) {  
-   k = m;
-  SETLENGTH(modelspace, nUnique);
-  SETLENGTH(logmarg, nUnique);
-  SETLENGTH(modelprobs, nUnique);
-  SETLENGTH(priorprobs, nUnique);
-  SETLENGTH(sampleprobs, nUnique);
-  SETLENGTH(counts, nUnique);
-  SETLENGTH(beta, nUnique);
-  SETLENGTH(se, nUnique);
-  SETLENGTH(mse, nUnique);
-  SETLENGTH(shrinkage, nUnique);
-  SETLENGTH(modeldim, nUnique);
-  SETLENGTH(R2, nUnique);
-  SETLENGTH(rank, nUnique);
+ /*. for when add heredity will need to use resizevector() as with Growable MCMC
+ if (m < nModels) {  
+  nModels  = m;
+
  }
 */  
- 
 
-  // Rprintf("Done with sampling - summaries m = %ld k = %ld \n", m, k);
-  compute_modelprobs(modelprobs, logmarg, priorprobs,k);
-  compute_margprobs(modelspace, modeldim, modelprobs, probs, k, p);
+  compute_sampleprobs_modelspace_Bernoulli(modelspace, modeldim, sampleprobs, 
+                                           Rprobs, nModels, p);   
+  
+
+ double eta = 1.0, NC = 0.0;
+ 
+  switch (INTEGER(RFPS)[0]) {
+    case 1:
+      compute_modelprobs_Bayes_HT(modelprobs, Rlogmarg, priorprobs, sampleprobs, nModels, &eta, &NC);
+    break;
+    default:
+      compute_modelprobs(modelprobs, Rlogmarg, priorprobs,nModels);
+    break;
+  }
+  switch (INTEGER(RFPS)[0]) {
+  case 1:
+    compute_margprobs_Bayes_BAS_MCMC(modelspace, modeldim, modelprobs, Rprobs, sampleprobs,nModels, p, eta,  NC);
+    break;
+  default:
+    compute_margprobs(modelspace, modeldim, modelprobs, probs, nModels, p);
+  break;
+  }
+ 
+  
 
   SET_VECTOR_ELT(ANS, 0, Rprobs);
   SET_STRING_ELT(ANS_names, 0, mkChar("probne0"));
@@ -418,7 +425,7 @@ SEXP mcmcbas(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeldim,
   SET_VECTOR_ELT(ANS, 1, modelspace);
   SET_STRING_ELT(ANS_names, 1, mkChar("which"));
 
-  SET_VECTOR_ELT(ANS, 2, logmarg);
+  SET_VECTOR_ELT(ANS, 2, Rlogmarg);
   SET_STRING_ELT(ANS_names, 2, mkChar("logmarg"));
 
   SET_VECTOR_ELT(ANS, 3, modelprobs);
