@@ -36,25 +36,97 @@ extern SEXP sampleworep_new(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit,
 	int nModels=LENGTH(Rmodeldim);
 	int pivot = LOGICAL(Rpivot)[0];
 	double tol = REAL(Rtol)[0];
+	
+	
+	int nModels0 = LENGTH(Rmodeldim);;  // initial guess on number of models to return
+	int nUnique = nModels0;
+
+	SEXP ANS = PROTECT(allocVector(VECSXP, 14)); ++nProtected;
+	SEXP ANS_names = PROTECT(allocVector(STRSXP, 14)); ++nProtected;
+	
+	SEXP Rprobs = duplicate(Rprobinit); 
+	SET_VECTOR_ELT(ANS, 0, Rprobs);
+	SET_STRING_ELT(ANS_names, 0, mkChar("probne0"));
+	
+	SEXP modelspace = allocVector(VECSXP, nModels0); 
+	SET_VECTOR_ELT(ANS, 1, modelspace);
+	SET_STRING_ELT(ANS_names, 1, mkChar("which"));
+	
+	SEXP Rlogmarg = allocVector(REALSXP, nModels0); 
+	SET_VECTOR_ELT(ANS, 2, Rlogmarg);
+	SET_STRING_ELT(ANS_names, 2, mkChar("logmarg"));
+	
+	SEXP modelprobs = allocVector(REALSXP, nModels0);  
+	SET_VECTOR_ELT(ANS, 3, modelprobs);
+	SET_STRING_ELT(ANS_names, 3, mkChar("postprobs"));
+	
+	SEXP priorprobs = allocVector(REALSXP, nModels0); 
+	SET_VECTOR_ELT(ANS, 4, priorprobs);
+	SET_STRING_ELT(ANS_names, 4, mkChar("priorprobs"));
+	
+	SEXP sampleprobs = allocVector(REALSXP, nModels0); 
+	SET_VECTOR_ELT(ANS, 5, sampleprobs);
+	SET_STRING_ELT(ANS_names, 5, mkChar("sampleprobs"));
+	
+	SEXP mse = allocVector(REALSXP, nModels0); 
+	SET_VECTOR_ELT(ANS, 6, mse);
+	SET_STRING_ELT(ANS_names, 6, mkChar("mse"));
+	
+	SEXP beta = allocVector(VECSXP, nModels0); 
+	SET_VECTOR_ELT(ANS, 7, beta);
+	SET_STRING_ELT(ANS_names, 7, mkChar("mle"));
+	
+	SEXP se = allocVector(VECSXP, nModels0);
+	SET_VECTOR_ELT(ANS, 8, se);
+	SET_STRING_ELT(ANS_names, 8, mkChar("mle.se"));
+	
+	SEXP shrinkage = allocVector(REALSXP, nModels0); 
+	SET_VECTOR_ELT(ANS, 9, shrinkage);
+	SET_STRING_ELT(ANS_names, 9, mkChar("shrinkage"));
+	
+	SEXP modeldim =  allocVector(INTSXP, nModels0); 
+	SET_VECTOR_ELT(ANS, 10, modeldim);
+	SET_STRING_ELT(ANS_names, 10, mkChar("size"));
+	
+	SEXP R2 = allocVector(REALSXP, nModels0); 
+	SET_VECTOR_ELT(ANS, 11, R2);
+	SET_STRING_ELT(ANS_names, 11, mkChar("R2"));
+	
+	SEXP rank = allocVector(INTSXP, nModels0); 
+	SET_VECTOR_ELT(ANS, 12, rank);
+	SET_STRING_ELT(ANS_names, 12, mkChar("rank"));
+	
+	
+	SEXP NumUnique = allocVector(INTSXP, 1); 
+	SET_VECTOR_ELT(ANS, 13, NumUnique);
+	SET_STRING_ELT(ANS_names, 13, mkChar("n.Unique"));
+	
+	setAttrib(ANS, R_NamesSymbol, ANS_names);
+	
+	
 
 	//  Rprintf("Allocating Space for %d Models\n", nModels) ;
+	/* Allocate Space for Results  R < 4.5.2
 	SEXP ANS = PROTECT(allocVector(VECSXP, 13)); ++nProtected;
 	SEXP ANS_names = PROTECT(allocVector(STRSXP, 13)); ++nProtected;
 	SEXP Rprobs = PROTECT(duplicate(Rprobinit)); ++nProtected;
-	SEXP R2 = PROTECT(allocVector(REALSXP, nModels)); ++nProtected;
-	SEXP shrinkage = PROTECT(allocVector(REALSXP, nModels)); ++nProtected;
-	SEXP modelspace = PROTECT(allocVector(VECSXP, nModels)); ++nProtected;
-	SEXP modeldim =  PROTECT(duplicate(Rmodeldim)); ++nProtected;
-	SEXP rank =  PROTECT(duplicate(Rmodeldim)); ++nProtected;
-	SEXP beta = PROTECT(allocVector(VECSXP, nModels)); ++nProtected;
-	SEXP se = PROTECT(allocVector(VECSXP, nModels)); ++nProtected;
-	SEXP mse = PROTECT(allocVector(REALSXP, nModels)); ++nProtected;
-	SEXP modelprobs = PROTECT(allocVector(REALSXP, nModels)); ++nProtected;
-	SEXP priorprobs = PROTECT(allocVector(REALSXP, nModels)); ++nProtected;
-	SEXP logmarg = PROTECT(allocVector(REALSXP, nModels)); ++nProtected;
-	SEXP sampleprobs = PROTECT(allocVector(REALSXP, nModels)); ++nProtected;
+	
 
-	double *Xwork, *Ywork, *wts, *probs, shrinkage_m, mse_m, R2_m, RSquareFull, Rbestmarg, logmargy;
+	  SEXP R2 = PROTECT(allocVector(REALSXP, nModels)); ++nProtected;
+	  SEXP shrinkage = PROTECT(allocVector(REALSXP, nModels)); ++nProtected;
+	  SEXP modelspace = PROTECT(allocVector(VECSXP, nModels)); ++nProtected;
+	  SEXP modeldim =  PROTECT(allocVector(VECSXP, nModels)); ++nProtected;
+	  SEXP rank =  PROTECT(allocVector(VECSXP, nModels)); ++nProtected;
+	  SEXP beta = PROTECT(allocVector(VECSXP, nModels)); ++nProtected;
+	  SEXP se = PROTECT(allocVector(VECSXP, nModels)); ++nProtected;
+	  SEXP mse = PROTECT(allocVector(REALSXP, nModels)); ++nProtected;
+	  SEXP modelprobs = PROTECT(allocVector(REALSXP, nModels)); ++nProtected;
+	  SEXP priorprobs = PROTECT(allocVector(REALSXP, nModels)); ++nProtected;
+	  SEXP Rlogmarg = PROTECT(allocVector(REALSXP, nModels)); ++nProtected;
+	  SEXP sampleprobs = PROTECT(allocVector(REALSXP, nModels)); ++nProtected;
+*/
+
+	double *Xwork, *Ywork, *wts, *probs, shrinkage_m, mse_m, R2_m, RSquareFull, Rbestmarg, logmarg_m;
 	int i, *model_m, *bestmodel, rank_m;
 
 	//get dimsensions of all variables
@@ -100,7 +172,7 @@ extern SEXP sampleworep_new(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit,
 
 	int m = 0;
 	bestmodel = INTEGER(Rbestmodel_new);
-	REAL(logmarg)[m] = 0.0;
+	REAL(Rlogmarg)[m] = 0.0;
 	INTEGER(modeldim)[m] = 0;
 
 	for (i = n; i < p; i++)  {
@@ -130,20 +202,21 @@ extern SEXP sampleworep_new(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit,
                  pmodel, p, nobs, m, &mse_m, &rank_m, pivot, tol);
 	INTEGER(rank)[m] = rank_m;
 
-	gexpectations(p, rank_m, nobs, R2_m, alpha, INTEGER(method)[0], RSquareFull, SSY, &logmargy, &shrinkage_m);
+	gexpectations(p, rank_m, nobs, R2_m, alpha, INTEGER(method)[0], RSquareFull, SSY, &logmarg_m, &shrinkage_m);
 //	Rprintf("rank %d dim %d\n", rank_m, pmodel);
-//	gexpectations(p, pmodel, nobs, R2_m, alpha, INTEGER(method)[0], RSquareFull, SSY, &logmargy, &shrinkage_m);
+//	gexpectations(p, pmodel, nobs, R2_m, alpha, INTEGER(method)[0], RSquareFull, SSY, &logmarg_m, &shrinkage_m);
 
 //  check should this depend on rank or pmodel?
 	double prior_m  = compute_prior_probs(model,pmodel,p, modelprior, noInclusionIs1);
 
 
 
-	SetModel2(logmargy, shrinkage_m, prior_m, sampleprobs, logmarg, shrinkage, priorprobs, m);
-	SetModel_lm(Rcoef_m, Rse_m, Rmodel_m, mse_m, R2_m,	beta, se, modelspace, mse, R2, m);
-	//Rprintf("model %d max logmarg %lf\n", m, REAL(logmarg)[m]);
-  UNPROTECT(3);
-	Rbestmarg = REAL(logmarg)[m];
+//	SetModel2(logmarg_m, shrinkage_m, prior_m, sampleprobs, Rlogmarg, shrinkage, priorprobs, m);
+	SetModel_lm(logmarg_m, shrinkage_m, prior_m, sampleprobs, Rlogmarg, shrinkage, priorprobs,
+             Rcoef_m, Rse_m, Rmodel_m, mse_m, R2_m,	beta, se, modelspace, mse, R2, m);
+	//Rprintf("model %d max logmarg %lf\n", m, REAL(Rlogmarg)[m]);
+
+	Rbestmarg = REAL(Rlogmarg)[m];
 //  double *parents = REAL(Rparents);
 	int *modelwork= ivecalloc(p);
 	/*
@@ -186,25 +259,26 @@ extern SEXP sampleworep_new(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit,
                   pmodel, p, nobs, m, &mse_m, &rank_m, pivot, tol);
 		INTEGER(rank)[m] = rank_m;
 		// initialize
-		logmargy= 0.0;
+		logmarg_m= 0.0;
 		shrinkage_m = 1.0;
-		gexpectations(p, rank_m, nobs, R2_m, alpha, INTEGER(method)[0], RSquareFull, SSY, &logmargy, &shrinkage_m);
+		gexpectations(p, rank_m, nobs, R2_m, alpha, INTEGER(method)[0], RSquareFull, SSY, &logmarg_m, &shrinkage_m);
 //    Rprintf("rank %d dim %d\n", rank_m, pmodel);
-//		gexpectations(p, pmodel, nobs, R2_m, alpha, INTEGER(method)[0], RSquareFull, SSY, &logmargy, &shrinkage_m);
+//		gexpectations(p, pmodel, nobs, R2_m, alpha, INTEGER(method)[0], RSquareFull, SSY, &logmarg_m, &shrinkage_m);
 
 		prior_m = compute_prior_probs(model,pmodel,p, modelprior, noInclusionIs1);
-		SetModel2(logmargy, shrinkage_m, prior_m, sampleprobs, logmarg, shrinkage, priorprobs, m);
-		SetModel_lm(Rcoef_m, Rse_m, Rmodel_m, mse_m, R2_m,	beta, se, modelspace, mse, R2,m);
-	  UNPROTECT(3);
+//		SetModel2(logmarg_m, shrinkage_m, prior_m, sampleprobs, Rlogmarg, shrinkage, priorprobs, m);
+		SetModel_lm(logmarg_m, shrinkage_m, prior_m, sampleprobs, Rlogmarg, shrinkage, priorprobs,
+                Rcoef_m, Rse_m, Rmodel_m, mse_m, R2_m,	beta, se, modelspace, mse, R2,m);
+	  
 
 		REAL(sampleprobs)[m] = pigamma[0];
 
 		//update best model
-		if (REAL(logmarg)[m] > Rbestmarg) {
+		if (REAL(Rlogmarg)[m] > Rbestmarg) {
 			for (i=0; i < p; i++) {
 				bestmodel[i] = model[i];
 			}
-			Rbestmarg = REAL(logmarg)[m];
+			Rbestmarg = REAL(Rlogmarg)[m];
 		}
 
 		//update marginal inclusion probs
@@ -213,7 +287,7 @@ extern SEXP sampleworep_new(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit,
 			double rem = modf((double) m/(double) update, &mod);
 			if (rem  == 0.0) {
 				int mcurrent = m;
-				compute_modelprobs(modelprobs, logmarg, priorprobs,mcurrent);
+				compute_modelprobs(modelprobs, Rlogmarg, priorprobs,mcurrent);
 				compute_margprobs(modelspace, modeldim, modelprobs, probs, mcurrent, p);
 				if (update_probs(probs, vars, mcurrent, k, p) == 1) {
 				  //					Rprintf("Updating Model Tree %d \n", m);
@@ -222,7 +296,35 @@ extern SEXP sampleworep_new(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit,
 			}
 		}
 	}
-
+  nUnique = k;
+	if (m < nUnique) { 
+	  // Rprintf("allocated %d models but only %d sampled; resizing\n", nUnique, m); 
+	  // resize
+	  // consider using force.heredity
+	  nUnique = m;
+	  
+	  compute_modelprobs(modelprobs, Rlogmarg, priorprobs,nUnique);
+	  compute_margprobs(modelspace, modeldim, modelprobs, probs, nUnique, p);
+	  SET_VECTOR_ELT(ANS, 1, xlengthgets(modelspace, nUnique));
+	  SET_VECTOR_ELT(ANS, 2, xlengthgets(Rlogmarg, nUnique));
+	  SET_VECTOR_ELT(ANS, 3, xlengthgets(modelprobs, nUnique));
+	  SET_VECTOR_ELT(ANS, 4, xlengthgets(priorprobs, nUnique));
+	  SET_VECTOR_ELT(ANS, 5, xlengthgets(sampleprobs, nUnique));
+	  SET_VECTOR_ELT(ANS, 6, xlengthgets(mse, nUnique));
+	  SET_VECTOR_ELT(ANS, 7, xlengthgets(beta, nUnique));
+	  SET_VECTOR_ELT(ANS, 8, xlengthgets(se, nUnique));
+	  SET_VECTOR_ELT(ANS, 9, xlengthgets(shrinkage, nUnique));
+	  SET_VECTOR_ELT(ANS, 10, xlengthgets(modeldim, nUnique));
+	  SET_VECTOR_ELT(ANS, 11, xlengthgets(R2, nUnique));
+	  SET_VECTOR_ELT(ANS, 12, xlengthgets(rank, nUnique));
+	}
+	else {
+	  compute_modelprobs(modelprobs, Rlogmarg, priorprobs,nUnique);
+	  compute_margprobs(modelspace, modeldim, modelprobs, probs, nUnique, p);
+	}
+	
+	INTEGER(NumUnique)[0] = nUnique;
+/*	
   if (m < k) { 
  // warning("allocated %d models but only %d sampled; using SETLENGTH to resize\n", k, m); 
  // resize
@@ -230,7 +332,7 @@ extern SEXP sampleworep_new(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit,
     k = m;
 
     SETLENGTH(modelspace, m);
-    SETLENGTH(logmarg, m);
+    SETLENGTH(Rlogmarg, m);
     SETLENGTH(modelprobs, m);
     SETLENGTH(priorprobs, m);
     SETLENGTH(sampleprobs, m);
@@ -244,7 +346,7 @@ extern SEXP sampleworep_new(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit,
   }
 
 
-	compute_modelprobs(modelprobs, logmarg, priorprobs,k);
+	compute_modelprobs(modelprobs, Rlogmarg, priorprobs,k);
 	compute_margprobs(modelspace, modeldim, modelprobs, probs, k, p);
 
 	SET_VECTOR_ELT(ANS, 0, Rprobs);
@@ -253,7 +355,7 @@ extern SEXP sampleworep_new(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit,
 	SET_VECTOR_ELT(ANS, 1, modelspace);
 	SET_STRING_ELT(ANS_names, 1, mkChar("which"));
 
-	SET_VECTOR_ELT(ANS, 2, logmarg);
+	SET_VECTOR_ELT(ANS, 2, Rlogmarg);
 	SET_STRING_ELT(ANS_names, 2, mkChar("logmarg"));
 
 	SET_VECTOR_ELT(ANS, 3, modelprobs);
@@ -287,6 +389,8 @@ extern SEXP sampleworep_new(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit,
 	SET_STRING_ELT(ANS_names, 12, mkChar("rank"));
 
 	setAttrib(ANS, R_NamesSymbol, ANS_names);
+ 
+ */
 	PutRNGstate();
 
 	UNPROTECT(nProtected);
