@@ -29,6 +29,23 @@ test_that("bas.glm initprobs" , {
                        betaprior = bic.prior(), family = binomial(),
                        modelprior = uniform())
   expect_equal(pima_bas1$postprobs, pima_bas2$postprobs)
+  
+  expect_error(bas.glm(type ~ .,
+                       data = Pima.tr, subset = 1:6,
+                       method = "BAS",
+                       initprobs= "eplogp",
+                       betaprior = bic.prior(), family = binomial(),
+                       modelprior = uniform())
+  )
+  
+  Pima.tr$type = NA
+  expect_warning(expect_error(bas.glm(type ~ .,
+                       data = Pima.tr,
+                       method = "BAS",
+                       initprobs= "eplogp",
+                       betaprior = bic.prior(), family = binomial(),
+                       modelprior = uniform()))
+  )
 })
 
 test_that("GLM logit", {
@@ -143,15 +160,40 @@ test_that("missing data arg", {
 
 test_that("poisson regression", {
   data(crabs, package = "glmbb")
-  crabs.bas <- bas.glm(satell ~ color * spine * width + weight,
+  set.seed(1)
+  crabs.MCMCbas <- bas.glm(satell ~ color * spine * width + weight,
     data = crabs,
     family = poisson(),
     betaprior = EB.local(), modelprior = uniform(),
-    method = "MCMC", n.models = 1024, MCMC.iterations = 10000,
+    method = "MCMC+BAS", 
+    n.models = 1024, burnin = 1000, MCMC.iterations = 10000,
     prob.rw = .95
   )
-  expect_null(plot(crabs.bas))
-  expect_equal(0, sum(crabs.bas$shrinkage > 1))
+  
+  set.seed(1)
+  crabs.MCMC <- bas.glm(satell ~ color * spine * width + weight,
+                           data = crabs,
+                           family = poisson(),
+                           betaprior = EB.local(), modelprior = uniform(),
+                           method = "MCMC", 
+                           burnin = 1000, MCMC.iterations = 10000,
+                           prob.rw = .95, n.models = 6000
+  )
+  set.seed(1)
+  crabs.MCMCbas <- bas.glm(satell ~ color * spine * width + weight,
+                           data = crabs,
+                           family = poisson(),
+                           betaprior = EB.local(), modelprior = uniform(),
+                           method = "MCMC+BAS", 
+                           n.models = crabs.MCMC$n.models, burnin = 1000+10000+1, MCMC.iterations = 0,
+                           prob.rw = .95
+  )
+ 
+  expect_equal(0, sum(crabs.MCMC$shrinkage > 1))
+  expect_equal(crabs.MCMC$logmarg, crabs.MCMCbas$logmarg)
+
+# currently not equal but not used  
+#  expect_equal(crabs.MCMC$freq, crabs.MCMCbas$freq)  
 })
 
 test_that("glm_fit", {
@@ -405,11 +447,19 @@ test_that("include always MCMC", {
   data("Pima.tr", package="MASS")
   pima_BAS = bas.glm(type ~ .,
                        data = Pima.tr, method = "MCMC",
-                       include.always = ~ bp,
+                       include.always = type ~ bp, 
                        betaprior = g.prior(g=100), family = binomial(),
                        modelprior = beta.binomial(1, 1))
   x = pima_BAS$probne0[match(c("Intercept", "bp") ,pima_BAS$namesx)]
   expect_equal(2, sum(x), tolerance=.002)
+  
+  expect_equal(0, sum(pima_BAS$R2 < 0))
+  
+ expect_warning(bas.glm(type ~ poly(bp,2),
+                     data = Pima.tr, method = "BAS",
+                     force.heredity = TRUE, bestmodel = c(1,0,1),
+                     betaprior = g.prior(g=100), family = binomial(),
+                     modelprior = beta.binomial(1, 1)))
 
 #  pima_BAS = bas.glm(type ~ .,
 #                     data = Pima.tr, method = "BAS",
@@ -420,8 +470,8 @@ test_that("include always MCMC", {
 ##  check why method='BAS' does not have 1.0 for keep.
 })
 
-# FIXED issue #35
-test_that("MCMC+BAS: missing MCMC.iterations and n.models arg", {
+# FIXED issue #35  missing MCMC.iterations and n.models arg
+test_that("MCMC+BAS", {
   data(Pima.tr, package = "MASS")
   set.seed(1)
   pima_BAS <- bas.glm(type ~ .,
@@ -447,6 +497,14 @@ test_that("MCMC+BAS: missing MCMC.iterations and n.models arg", {
   expect_equal(pima_BAS$probne0, pima_2$probne0)
   expect_equal(pima_BAS$n.models, pima_1$n.models)
   expect_equal(pima_BAS$n.models, pima_2$n.models)
+  
+  set.seed(42)
+  pima_BAS <- bas.glm(type ~ .,
+                      data = Pima.tr, method = "MCMC+BAS",
+                      betaprior = bic.prior(),
+                      family = binomial(),
+                      modelprior = uniform(), burnin.it = 5, MCMC.iterations = 5,  update =  50)
+  expect_equal(6, sum(pima_BAS$freq))
 })
 
 # issue 38 (in progress)  check that it works with other prior
@@ -472,6 +530,20 @@ test_that("herdity and BAS", {
   expect_equal(0L, sum(pima_BAS_no$probne0[-1] > 1.0))
   expect_equal(pima_BAS$probne0, pima_BAS_no$probne0)
   expect_equal(0L, sum(duplicated(pima_BAS$which)))
+  
+  pima_BAS <-  bas.glm(type ~ (bp + glu + npreg),
+                       data = Pima.tr, method = "BAS",
+                       betaprior = bic.prior(),
+                       family = binomial(), update=NULL,
+                       modelprior =uniform(),
+                       force.heredity=TRUE)
+  pima_BAS_no <-  bas.glm(type ~ (bp + glu + npreg),
+                          data = Pima.tr, method = "BAS",
+                          betaprior = bic.prior(),
+                          family = binomial(),  update=NULL,
+                          modelprior =uniform(),
+                          force.heredity=FALSE)
+  expect_equal(pima_BAS$probne0, pima_BAS_no$probne0)
 })
 
 # issue 55 in progress
@@ -573,3 +645,36 @@ test_that("gamma regression coef", {
   expect_warning(coef(wafer_bas), NA)
   
 })
+
+# test Binomial Model with Successes and Failures. Issue #94
+
+test_that("Binomial Model with Successes/Failures", { 
+Gegevens <- data.frame(
+  Jaar      = seq(from=2020,to=2024),
+  Totaal = c(29,18,19,15,15),
+  FeitenBeoordeling = c(14,12,13,7,7),
+  Beoordeling = c(13,8,10,5,5),
+  MentionFB = cbind(c(14,12,13,7,7), c(15,6,6,8,8)),
+  MentionB  = cbind(c(13,8,10,5,5), c(16,10,9,10,10))
+)
+
+bin.mod <- glm(cbind(Gegevens$MentionFB.1,Gegevens$MentionFB.2) ~
+                 Jaar, data=Gegevens,
+               family=binomial(link = "logit"))
+bin.fit = glm.fit(y = cbind(Gegevens$MentionFB.1,Gegevens$MentionFB.2),
+                  x = cbind(1.0, Gegevens$Jaar), 
+                  family=binomial(link = "logit"))
+bas.fit = bayesglm.fit(y = cbind(Gegevens$MentionFB.1,Gegevens$MentionFB.2),
+                  x = cbind(1.0, Gegevens$Jaar), 
+                  family=binomial(link = "logit"))
+expect_equal(as.numeric(bin.mod$coefficients), bas.fit$coefficients) #no names in bas.fit
+# ISSUE #94
+expect_no_error(bas.glm(cbind(Gegevens$MentionFB.1,Gegevens$MentionFB.2) ~
+                          Jaar, data=Gegevens,
+                        family=binomial(link = "logit"), betaprior = bic.prior()))
+  bin.mod.bas <- bas.glm(cbind(Gegevens$MentionFB.1,Gegevens$MentionFB.2) ~
+                 Jaar, data=Gegevens, include.always = ~ Jaar,
+                 family=binomial(link = "logit"), betaprior = bic.prior())
+  expect_equal(as.numeric(bin.mod$coefficients), unlist(bin.mod.bas$mle))
+
+})
\ No newline at end of file
